import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  scaleLinear,
  extent,
  max,
  format,
  scaleOrdinal,
} from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { ColorLegend } from './ColorLegend';

const width = window.innerWidth;
const height = window.innerHeight;
const margin = {
  top: 20,
  right: 200,
  bottom: 65,
  left: 90,
};
const innerWidth = width - margin.left - margin.right;
const innerHeight = height - margin.top - margin.bottom;

const xAxisLabelOffset = 50;
const yAxisLabelOffset = 40;

const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }
  console.log(data[0]);

  const xValue = (d) => d.petal_length;
  const xAxisLabel = 'Petal length';

  const yValue = (d) => d.sepal_width;
  const yAxisLabel = 'Sepal width';

  const colorValue = (d) => d.species;
  const colorLegendLabel = 'Species';

  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(['#E6842A', '#137B80', '#8E6C8A']);

  const circleRadius = 7;

  console.log(colorScale.domain());

  const siFormat = format('.2s');
  const xAxisTickFormat = (n) =>
    siFormat(n).replace('G', 'B');

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  return (
    <svg width={width} height={height}>
      <g
        className="tick"
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <AxisBottom
          tickFormat={xAxisTickFormat}
          xScale={xScale}
          innerHeight={innerHeight}
          tickOffset={5}
        />
        <AxisLeft
          yScale={yScale}
          innerWidth={innerWidth}
          tickOffset={-5}
        />
        <text
          className="axis-label"
          x={innerWidth / 2}
          textAnchor="middle"
          y={innerHeight + xAxisLabelOffset}
        >
          {xAxisLabel}
        </text>
        <text
          transform={`rotate(-90) translate(${
            -innerHeight / 2
          }, ${-yAxisLabelOffset})`}
          className="axis-label"
          text-anchor="middle"
        >
          {yAxisLabel}
        </text>
        <g
          transform={`translate(${innerWidth + 60}, ${60})`}
        >
          <text 
            x={30}
            y={-25}
            className="axis-label" 
            textAnchor="middle"
          >
            {colorLegendLabel}
          </text>
          <ColorLegend
            tickSpacing={22}
            tickSize={circleRadius}
            tickTextOffset={12}
            colorScale={colorScale}
          />
        </g>
        <Marks
          data={data}
          xScale={xScale}
          xValue={xValue}
          yScale={yScale}
          yValue={yValue}
          colorScale={colorScale}
          colorValue={colorValue}
          tooltipFormat={xAxisTickFormat}
          circleRadius={circleRadius}
        />
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
